@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_FLAC_PARSE_H__
@@ -76,6 +76,8 @@ struct _GstFlacParse {
   guint64 sample_number;
   gboolean strategy_checked;
 
+  gboolean sent_codec_tag;
+
   GstTagList *tags;
   GstToc *toc;
 
@@ -83,6 +85,10 @@ struct _GstFlacParse {
   GstBuffer *seektable;
 
   gboolean force_variable_block_size;
+
+  gsize header_size;
+  gsize byte_offset;
+  gboolean byte_mode;
 };
 
 struct _GstFlacParseClass {
