@@ -265,10 +265,6 @@ gst_image_freeze_sink_getcaps (GstImageFreeze * self, GstCaps * filter)
   GstPad *pad;
 
   pad = self->sinkpad;
-  ret = gst_pad_get_current_caps (pad);
-  if (ret != NULL) {
-    goto done;
-  }
 
   if (filter) {
     filter = gst_caps_copy (filter);
@@ -292,7 +288,6 @@ gst_image_freeze_sink_getcaps (GstImageFreeze * self, GstCaps * filter)
   ret = gst_caps_make_writable (ret);
   gst_image_freeze_remove_fps (self, ret);
 
-done:
   GST_LOG_OBJECT (pad, "Returning caps: %" GST_PTR_FORMAT, ret);
 
   return ret;
@@ -491,6 +486,13 @@ gst_image_freeze_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       ret = TRUE;
       break;
     }
+    case GST_QUERY_LATENCY:
+      /* This will only return an accurate latency for the first buffer since
+       * all further buffers outputted by us are just copies of that one, and
+       * the latency is 0 in that case. However, latency changes are not
+       * straightforward, so let's do the conservative fix for now. */
+      ret = gst_pad_query_default (pad, parent, query);
+      break;
     default:
       ret = FALSE;
       break;
