@@ -113,6 +113,8 @@ void          atoms_context_free (AtomsContext *context);
 
 /* atom defs and functions */
 
+typedef struct _AtomInfo AtomInfo;
+
 /*
  * Used for storing time related values for some atoms.
  */
@@ -701,6 +703,7 @@ typedef struct _AtomTRAK
   Atom header;
 
   AtomTKHD tkhd;
+  AtomInfo *tapt;
   AtomEDTS *edts;
   AtomMDIA mdia;
   AtomUDTA udta;
@@ -893,13 +896,12 @@ typedef guint64 (*AtomFreeFunc) (Atom *atom);
  * All we need are the two functions (copying it to an array
  * for serialization and the memory releasing function).
  */
-typedef struct _AtomInfo
+struct _AtomInfo
 {
   Atom *atom;
   AtomCopyDataFunc copy_data_func;
   AtomFreeFunc free_func;
-} AtomInfo;
-
+};
 
 guint64    atom_copy_data              (Atom *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
@@ -916,6 +918,7 @@ void       atom_trak_add_samples       (AtomTRAK * trak, guint32 nsamples, guint
                                         gint64 pts_offset);
 void       atom_trak_set_elst_entry    (AtomTRAK * trak, gint index, guint32 duration,
                                         guint32 media_time, guint32 rate);
+void       atom_trak_edts_clear        (AtomTRAK * trak);
 guint32    atom_trak_get_timescale     (AtomTRAK *trak);
 guint32    atom_trak_get_id            (AtomTRAK * trak);
 void       atom_stbl_add_samples       (AtomSTBL * stbl, guint32 nsamples,
@@ -1056,6 +1059,9 @@ AtomInfo *   build_jp2x_extension        (const GstBuffer * prefix);
 AtomInfo *   build_fiel_extension        (GstVideoInterlaceMode mode, GstVideoFieldOrder order);
 AtomInfo *   build_colr_extension        (const GstVideoColorimetry *colorimetry, gboolean is_mp4);
 AtomInfo *   build_clap_extension        (gint width_n, gint width_d, gint height_n, gint height_d, gint h_off_n, gint h_off_d, gint v_off_n, gint v_off_d);
+AtomInfo *   build_tapt_extension        (gint clef_width, gint clef_height, gint prof_width, gint prof_height, gint enof_width, gint enof_height);
+
+
 AtomInfo *   build_ac3_extension         (guint8 fscod, guint8 bsid,
                                           guint8 bsmod, guint8 acmod,
                                           guint8 lfe_on, guint8 bitrate_code);
