@@ -179,6 +179,10 @@ G_BEGIN_DECLS
 #define FOURCC_pasp     GST_MAKE_FOURCC('p','a','s','p')
 #define FOURCC_colr     GST_MAKE_FOURCC('c','o','l','r')
 #define FOURCC_clap     GST_MAKE_FOURCC('c','l','a','p')
+#define FOURCC_tapt     GST_MAKE_FOURCC('t','a','p','t')
+#define FOURCC_clef     GST_MAKE_FOURCC('c','l','e','f')
+#define FOURCC_prof     GST_MAKE_FOURCC('p','r','o','f')
+#define FOURCC_enof     GST_MAKE_FOURCC('e','n','o','f')
 #define FOURCC_fiel     GST_MAKE_FOURCC('f','i','e','l')
 #define FOURCC_pcst     GST_MAKE_FOURCC('p','c','s','t')
 #define FOURCC_pgap     GST_MAKE_FOURCC('p','g','a','p')
